@@ -232,7 +232,7 @@ find_channel (server *serv, char *chan)
 	while (list)
 	{
 		sess = list->data;
-		if ((!serv || serv == sess->server) && sess->type != SESS_DIALOG)
+		if ((!serv || serv == sess->server) && sess->type == SESS_CHANNEL)
 		{
 			if (!serv->p_cmp (chan, sess->channel))
 				return sess;
@@ -286,12 +286,16 @@ lag_check (void)
 								 NULL, NULL, 0);
 				if (prefs.hex_net_auto_reconnect)
 					serv->auto_reconnect (serv, FALSE, -1);
-			} else if (!serv->lag_sent)
+			} else
 			{
 				snprintf (tbuf, sizeof (tbuf), "LAG%lu", tim);
 				serv->p_ping (serv, "", tbuf);
-				serv->lag_sent = tim;
-				fe_set_lag (serv, -1);
+				
+				if (!serv->lag_sent)
+				{
+					serv->lag_sent = tim;
+					fe_set_lag (serv, -1);
+				}
 			}
 		}
 		list = list->next;
@@ -447,6 +451,7 @@ irc_init (session *sess)
 
 	if (arg_command != NULL)
 	{
+		handle_command (sess, arg_command, FALSE);
 		g_free (arg_command);
 	}
 
@@ -523,6 +528,8 @@ new_ircwindow (server *serv, char *name, int type, int focus)
 	irc_init (sess);
 	chanopt_load (sess);
 	scrollback_load (sess);
+	if (sess->scrollwritten && sess->scrollback_replay_marklast)
+		sess->scrollback_replay_marklast (sess);
 	plugin_emit_dummy_print (sess, "Open Context");
 
 	return sess;
@@ -735,7 +742,7 @@ static char defaultconf_commands[] =
 	"NAME WII\n"			"CMD quote WHOIS %2 %2\n\n";
 
 static char defaultconf_urlhandlers[] =
-		"NAME Open Link in Private Firefox Window\n"		"CMD !firefox -private %s\n\n";
+		"NAME Open Link in a new Firefox Window\n"		"CMD !firefox -new-window %s\n\n";
 
 #ifdef USE_SIGACTION
 /* Close and open log files on SIGUSR1. Usefull for log rotating */
@@ -1088,7 +1095,7 @@ main (int argc, char *argv[])
 	{
 		/* this is probably the first run */
 		load_default_config ();
-		make_config_dirs (); /* FIXME: if this fail display an error (?) */
+		make_config_dirs ();
 		make_dcc_dirs ();
 	}
 
@@ -1113,6 +1120,18 @@ main (int argc, char *argv[])
 
 	fe_init ();
 
+	/* This is done here because cfgfiles.c is too early in
+	* the startup process to use gtk functions. */
+	if (g_access (get_xdir (), W_OK) != 0)
+	{
+		char buf[2048];
+
+		g_snprintf (buf, sizeof(buf),
+			_("You do not have write access to %s. Nothing from this session can be saved."),
+			get_xdir ());
+		fe_message (buf, FE_MSG_ERROR);
+	}
+
 #ifndef WIN32
 #ifndef __EMX__
 	/* OS/2 uses UID 0 all the time */
