@@ -17,6 +17,8 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
  */
 
+#include "proto-irc.h"
+
 #ifndef HEXCHAT_MODES_H
 #define HEXCHAT_MODES_H
 
@@ -24,8 +26,9 @@ int is_channel (server *serv, char *chan);
 char get_nick_prefix (server *serv, unsigned int access);
 unsigned int nick_access (server *serv, char *nick, int *modechars);
 int mode_access (server *serv, char mode, char *prefix);
-void inbound_005 (server *serv, char *word[]);
-void handle_mode (server *serv, char *word[], char *word_eol[], char *nick, int numeric_324);
+void inbound_005 (server *serv, char *word[], const message_tags_data *tags_data);
+void handle_mode (server *serv, char *word[], char *word_eol[], char *nick,
+						int numeric_324, const message_tags_data *tags_data);
 void send_channel_modes (session *sess, char *tbuf, char *word[], int start, int end, char sign, char mode, int modes_per_line);
 
 #endif
