@@ -57,9 +57,6 @@ static const struct defaultserver def[] =
 	{0,			"irc2.2ch.net"},
 	{0,			"irc.juggler.jp"},
 
-/*	{"7-indonesia",	0},
-	{0,			"irc.7-indonesia.org"},*/
-
 	{"AccessIRC",	0},
 	{0,			"irc.accessirc.net"},
 	{0,			"eu.accessirc.net"},
@@ -89,19 +86,18 @@ static const struct defaultserver def[] =
 	{0,			"irc.5ci.net"},
 	{0,			"irc.kis.lt"},
 
-	{"AlphaChat",	0},
+	{"AlphaChat",	0, 0, 0, LOGIN_SASL},
 	{0,			"irc.alphachat.net"},
 	{0,			"na.alphachat.net"},
 	{0,			"eu.alphachat.net"},
 	{0,			"au.alphachat.net"},
 	{0,			"za.alphachat.net"},
 
-/*	{"AmigaNet",	0},
-	{0,			"irc.amiganet.org"},
-	{0,			"us.amiganet.org"},
-	{0,			"uk.amiganet.org"},
-	{0,			"no.amiganet.org"},
-	{0,			"au.amiganet.org"},*/
+	{"Anthrochat", 0},
+#ifdef USE_OPENSSL
+	{0,			"irc.anthrochat.net/+6697"},
+#endif
+	{0,			"irc.anthrochat.net"},
 
 	{"ARCNet",	0},
 	{0,			"se1.arcnet.vapor.com"},
@@ -116,23 +112,13 @@ static const struct defaultserver def[] =
 	{0,			"nl3.arcnet.vapor.com"},
 	{0,			"uk1.arcnet.vapor.com"},
 	{0,			"uk2.arcnet.vapor.com"},
-/*	{0,			"uk3.arcnet.vapor.com"},*/
 	{0,			"fr1.arcnet.vapor.com"},
 
-/*	{"AstroLink",	0},
-	{0,			"irc.astrolink.org"},*/
-
 	{"AustNet",	0},
 	{0,			"au.austnet.org"},
 	{0,			"us.austnet.org"},
 	{0,			"ca.austnet.org"},
 
-/*	{"AxeNet",	0},
-	{0,			"irc.axenet.org"},
-	{0,			"angel.axenet.org"},
-	{0,			"energy.axenet.org"},
-	{0,			"python.axenet.org"},*/
-
 	{"AzzurraNet",	0},
 	{0,			"irc.azzurra.org"},
 	{0,			"crypto.azzurra.org"},
@@ -140,6 +126,12 @@ static const struct defaultserver def[] =
 	{"Beirut", 0},
 	{0,			"irc.beirut.com"},
 
+	{"Canternet", 0, 0, 0, LOGIN_SASL},
+#ifdef USE_OPENSSL
+	{0, 		"irc.canternet.org/+6697"},
+#endif
+	{0,			"irc.canternet.org"},
+
 	{"Chat4all", 0},
 #ifdef USE_OPENSSL
 	{0,			"irc.chat4all.org/+7001"},
@@ -149,7 +141,7 @@ static const struct defaultserver def[] =
 	{"ChattingAway", 0},
 	{0,			"irc.chattingaway.com"},
 
-	{"ChatJunkies",	0, "#xchat"},
+	{"ChatJunkies",	0},
 	{0,			"irc.chatjunkies.org"},
 	{0,			"nl.chatjunkies.org"},
 
@@ -157,18 +149,9 @@ static const struct defaultserver def[] =
 	{0,			"US.ChatNet.Org"},
 	{0,			"EU.ChatNet.Org"},
 
-/*	{"ChatSociety", 0},
-	{0,			"us.chatsociety.net"},
-	{0,			"eu.chatsociety.net"},*/
-
 	{"ChatSpike", 0},
 	{0,			"irc.chatspike.net"},
 
-/*	{"CoolChat",	0},
-	{0,			"irc.coolchat.net"},
-	{0,			"unix.coolchat.net"},
-	{0,			"toronto.coolchat.net"},*/
-
 	{"Criten", 0},
 	{0,			"irc.criten.net"},
 	{0,			"irc.eu.criten.net"},
@@ -183,7 +166,7 @@ static const struct defaultserver def[] =
 	{0,			"nc.d-t-net.de"},
 	{0,			"wakka.d-t-net.de"},
 
-	{"DarkMyst", 0},
+	{"DarkMyst", 0, 0, 0, LOGIN_SASL},
 	{0,			"irc.darkmyst.org"},
 
 	{"DeepIRC", 0},
@@ -198,12 +181,18 @@ static const struct defaultserver def[] =
 	{0,			"irc.Qeast.net"},
 	{0,			"irc.efnet.pl"},
 	{0,			"efnet.demon.co.uk"},
-/*	{0,			"irc.lagged.org"},*/
 	{0,			"irc.lightning.net"},
 	{0,			"irc.mindspring.com"},
 	{0,			"irc.easynews.com"},
 	{0,			"irc.servercentral.net"},
 
+	{"ElectroCode",		0},
+#ifdef USE_OPENSSL
+
+	{0,			"irc.electrocode.net/+6697"},
+#endif
+	{0,			"irc.electrocode.net"},
+
 	{"EnterTheGame",	0},
 	{0,			"IRC.EnterTheGame.Com"},
 
@@ -219,7 +208,7 @@ static const struct defaultserver def[] =
 	{0,			"irc6.entropynet.net"},
 #endif
 
-	{"EsperNet",	0},
+	{"EsperNet", 0, 0, 0, LOGIN_SASL},
 #ifdef USE_OPENSSL
 	{0,			"irc.esper.net/+6697"},
 #endif
@@ -239,14 +228,11 @@ static const struct defaultserver def[] =
 	{"EuropNet", 0},
 	{0,			"irc.europnet.org"},
 
-/*	{"EU-IRC",	0},
-	{0,			"irc.eu-irc.net"},*/
-
 	{"FDFNet",	0},
 	{0,			"irc.fdfnet.net"},
 	{0,			"irc.eu.fdfnet.net"},
 
-	{"FEFNet",	0},
+	{"FEFNet", 0, 0, 0, LOGIN_SASL},
 	{0,			"irc.fef.net"},
 	{0,			"irc.ggn.net"},
 	{0,			"irc.vendetta.com"},
@@ -259,52 +245,33 @@ static const struct defaultserver def[] =
 	/* irc. points to chat. but many users and urls still reference it */
 	{0,				"irc.freenode.net"},
 
-/*	{"Freeworld",	0},
-	{0,			"kabel.freeworld.nu"},
-	{0,			"irc.freeworld.nu"},*/
-
-	{"FurryLand",	0},
+	{"Furnet",	0},
 #ifdef USE_OPENSSL
-	{0,				"irc.furryland.net/+6697"},
+	{0,			"irc.furnet.org/+6697"},
 #endif
-	{0,				"irc.furryland.net"},	
+	{0,			"irc.furnet.org"},
 
 	{"Fusion Latina",	0},
 	{0,					"irc.fusionlatina.org/2012"},
 
 	{"GalaxyNet",	0},
 	{0,			"irc.galaxynet.org"},
-/*	{0,			"sprynet.us.galaxynet.org"},
-	{0,			"atlanta.ga.us.galaxynet.org"},*/
 
 	{"GameSurge", 0},
 	{0,			"irc.gamesurge.net"},
 	
-/*	{"GamesNET",	0},
-	{0,				"irc.gamesnet.net"},
-	{0,				"irc.us.gamesnet.net"},
-	{0,				"east.us.gamesnet.net"},
-	{0,				"west.us.gamesnet.net"},
-	{0,				"irc.ca.gamesnet.net"},
-	{0,				"irc.eu.gamesnet.net"},*/
-
 	{"GeekShed",	0},
 	{0,			"irc.geekshed.net"},
 
 	{"German-Elite",	0},
 	{0,			"dominion.german-elite.net"},
 	{0,			"komatu.german-elite.net"},
-/*	{0,			"liberty.german-elite.net"},*/
 
 	{"GIMPNet",		0},
 	{0,			"irc.gimp.org"},
 	{0,			"irc.gnome.org"},
-/*	{0,			"irc.au.gimp.org"},*/
 	{0,			"irc.us.gimp.org"},
 
-/*	{"HabberNet",	0},
-	{0,			"irc.habber.net"},*/
-
 	{"Hashmark",	0},
 	{0,			"irc.hashmark.net"},
 
@@ -316,12 +283,12 @@ static const struct defaultserver def[] =
 	{0,			"irc.indirectirc.com/+6697"},
 #endif
 	{0,			"irc.indirectirc.com"},
-
-/*	{"Infinity-IRC",	0},
-	{0,			"Atlanta.GA.US.Infinity-IRC.Org"},
-	{0,			"Babylon.NY.US.Infinity-IRC.Org"},
-	{0,			"Sunshine.Ca.US.Infinity-IRC.Org"},
-	{0,			"IRC.Infinity-IRC.Org"},*/
+	
+	{"Interlinked", 0, 0, 0, LOGIN_SASL},
+#ifdef USE_OPENSSL
+	{0,			"irc.interlinked.me/+6697"},
+#endif
+	{0,			"irc.interlinked.me"},
 
 	{"IRCHighWay",	0},
 #ifdef USE_OPENSSL
@@ -340,7 +307,7 @@ static const struct defaultserver def[] =
 	{0,				"open.ircnet.net"},
 	{0,				"irc.de.ircnet.net"},
 	
-	{"IRCNode", 0},
+	{"IRCNode", 0, 0, 0, LOGIN_SASL},
 #ifdef USE_OPENSSL
 	{0,			"irc.ircnode.org/+6697"},
 #endif
@@ -349,12 +316,6 @@ static const struct defaultserver def[] =
 	{"Irctoo.net",	0},
 	{0,			"irc.irctoo.net"},
 
-	{"IronDust", 0},
-#ifdef USE_OPENSSL
-	{0,			"irc.irondust.net/+6697"},
-#endif
-	{0,			"irc.irondust.net"},
-
 	{"iZ-smart.net",	0},
 	{0,			"irc.iZ-smart.net/6666"},
 	{0,			"irc.iZ-smart.net/6667"},
@@ -363,10 +324,6 @@ static const struct defaultserver def[] =
 	{"Krstarica", 0},
 	{0,			"irc.krstarica.com"},
 
-	{"Librenet",	0},
-	{0,			"irc.librenet.net"},
-	{0,			"ielf.fr.librenet.net"},
-
 #ifdef USE_OPENSSL
 	{"LinkNet",	0},
 	{0,			"irc.link-net.org/+7000"},
@@ -376,22 +333,11 @@ static const struct defaultserver def[] =
 #ifdef USE_IPV6
 	{0,			"irc6.link-net.org/+7000"},
 #endif
-/*	{0,			"irc.no.link-net.org"},
-	{0,			"irc.gamesden.net.au"},
-	{0,			"irc.bahnhof.se"},
-	{0,			"irc.kinexuseurope.co.uk"},
-	{0,			"irc.gamiix.com"},*/
 #endif
 
-/*	{"Majistic",	0},
-	{0,			"irc.majistic.net"},*/
-
 	{"MindForge",	0},
 	{0,			"irc.mindforge.org"},
 
-/*	{"MintIRC",	0},
-	{0,			"irc.mintirc.net"},*/
-
 	{"MIXXnet",		0},
 	{0,			"irc.mixxnet.net"},
 
@@ -405,26 +351,11 @@ static const struct defaultserver def[] =
 	{0,			"universe.nevernet.net"},
 	{0,			"wayland.nevernet.net"},
 	{0,			"forte.nevernet.net"},
-
-	{"NixHelpNet",	0},
-	{0,			"irc.nixhelp.org"},
-	{0,			"us.nixhelp.org"},
-	{0,			"uk.nixhelp.org"},
-	{0,			"uk2.nixhelp.org"},
-	{0,			"uk3.nixhelp.org"},
-	{0,			"nl.nixhelp.org"},
-	{0,			"ca.ld.nixhelp.org"},
-	{0,			"us.co.nixhelp.org"},
-	{0,			"us.ca.nixhelp.org"},
-	{0,			"us.pa.nixhelp.org"},
-
-/*	{"NullusNet",	0},
-	{0,			"irc.nullus.net"},*/
 	
 	{"ObsidianIRC",  0},
 	{0,      "irc.obsidianirc.net"}, 
 
-	{"Oceanius", 0},
+	{"Oceanius", 0, 0, 0, LOGIN_SASL},
 	{0,			"irc.oceanius.com"},
 
 	{"OFTC",	0},
@@ -439,7 +370,7 @@ static const struct defaultserver def[] =
 	{"PIRC.PL",	0},
 	{0,			"irc.pirc.pl"},
 	
-	{"PonyChat",	0},
+	{"PonyChat", 0, 0, 0, LOGIN_SASL},
 #ifdef USE_OPENSSL
 	{0, 		"irc.ponychat.net/+6697"},
 #endif
@@ -466,26 +397,6 @@ static const struct defaultserver def[] =
 	{0,			"irc.de.quakenet.org"},
 	{0,			"irc.it.quakenet.org"},
 
-/*	{"RebelChat",	0},
-	{0,			"irc.rebelchat.org"},*/
-
-/*	{"Recycled-IRC",  0},
-	{0,			"irc.recycled-irc.org"},
-	{0,			"vermin.recycled-irc.org"},
-	{0,			"waste.recycled-irc.org"},
-	{0,			"lumber.recycled-irc.org"},
-	{0,			"trash.recycled-irc.org"},
-	{0,			"unwashed.recycled-irc.org"},
-	{0,			"garbage.recycled-irc.org"},
-	{0,			"dustbin.recycled-irc.org"},*/
-
-/*	{"RizeNET", 0},
-	{0,			"irc.rizenet.org"},
-	{0,			"omega.rizenet.org"},
-	{0,			"evelance.rizenet.org"},
-	{0,			"lisa.rizenet.org"},
-	{0,			"scott.rizenet.org"},*/
-
 	{"Rizon", 0},
 	{0,			"irc.rizon.net"},
 
@@ -504,6 +415,12 @@ static const struct defaultserver def[] =
 	{"SeilEn.de",	0},
 	{0,			"irc.seilen.de"},
 
+	{"SeionIRC", 0, 0, 0, LOGIN_SASL},
+#ifdef USE_OPENSSL
+	{0,			"irc.seion.us/+6697"},
+#endif
+	{0,			"irc.seion.us"},
+
 	{"Serenity-IRC",	0},
 	{0,			"irc.serenity-irc.net"},
 	{0,			"eu.serenity-irc.net"},
@@ -530,17 +447,12 @@ static const struct defaultserver def[] =
 	{"SolidIRC", 0},
 	{0,			"irc.solidirc.com"},
 
-	{"SorceryNet",	0},
+	{"SorceryNet", 0, 0, 0, LOGIN_SASL},
 	{0,			"irc.sorcery.net/9000"},
 	{0,			"irc.us.sorcery.net/9000"},
 	{0,			"irc.eu.sorcery.net/9000"},
-
-/*	{"Spidernet",	0},
-	{0,			"us.spidernet.org"},
-	{0,			"eu.spidernet.org"},
-	{0,			"irc.spidernet.org"},*/
 	
-	{"SpotChat", 0},
+	{"SpotChat", 0, 0, 0, LOGIN_SASL},
 #ifdef USE_OPENSSL
 	{0,			"irc.spotchat.org/+6697"},
 #endif
@@ -563,18 +475,33 @@ static const struct defaultserver def[] =
 	{0,			"irc.station51.net/+6697"},
 #endif
 	{0,			"irc.station51.net"},
-	
+
+	{"StormBit", 0, 0, 0, LOGIN_SASL},
+#ifdef USE_OPENSSL
+	{0,			"irc.stormbit.net/+6697"},
+#endif
+	{0,			"irc.stormbit.net"},
+
 	{"SwiftIRC", 0},
 #ifdef USE_OPENSSL
 	{0,			"irc.swiftirc.net/+6697"},
 #endif
 	{0,			"irc.swiftirc.net/6667"},
 
-	{"TinyCrab",			0},
-	{0,			"irc.tinycrab.net"},
+	{"synIRC", 0},
+#ifdef USE_OPENSSL
+	{0, "irc.synirc.net/+6697"},
+#endif
+	{0, "irc.synirc.net/6667"},
 
-/*	{"TNI3",			0},
-	{0,			"irc.tni3.com"},*/
+	{"Techman's World IRC",	0, 0, 0, LOGIN_SASL},
+#ifdef USE_OPENSSL
+	{0,			"irc.techmansworld.com/+6697"},
+#endif
+	{0,			"irc.techmansworld.com/6667"},
+
+	{"TinyCrab", 0, 0, 0, LOGIN_SASL},
+	{0,			"irc.tinycrab.net"},
 
 	{"TURLINet",			0},
 	{0,			"irc.turli.net"},
@@ -593,7 +520,7 @@ static const struct defaultserver def[] =
 	{0,			"irc.telecoms.bg"},
 	{0,			"irc.tu-varna.edu"},
 	
-	{"ValleyNode",	0},
+	{"ValleyNode", 0, 0, 0, LOGIN_SASL},
 	{0,			"irc.valleynode.net"},
 
 	{"Whiffle",	0},
@@ -609,15 +536,11 @@ static const struct defaultserver def[] =
 #endif
 	{0,			"irc.windfyre.net"},
 
-/*	{"Xentonix.net",	0},
-	{0,			"irc.xentonix.net"},*/
-
-/*	{"XWorld",	0},
-	{0,			"Buffalo.NY.US.XWorld.org"},
-	{0,			"Minneapolis.MN.US.Xworld.Org"},
-	{0,			"Rochester.NY.US.XWorld.org"},
-	{0,			"Bayern.DE.EU.XWorld.Org"},
-	{0,			"Chicago.IL.US.XWorld.Org"},*/
+	{"Xertion", 0, 0, 0, LOGIN_SASL},
+#ifdef USE_OPENSSL
+	{0,			"irc.xertion.org/+6697"},
+#endif
+	{0,			"irc.xertion.org"},
 
 	{0,0}
 };
@@ -925,7 +848,7 @@ servlist_favchan_find (ircnet *net, char *channel, int *pos)
 	while (list)
 	{
 		favchan = list->data;
-		if (strcmp (favchan->name, channel) == 0)
+		if (g_ascii_strcasecmp (favchan->name, channel) == 0)
 		{
 			if (pos)
 			{
