@@ -25,6 +25,7 @@
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
+#include <time.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 
@@ -32,7 +33,7 @@
 #include <sys/timeb.h>
 #include <process.h>
 #include <io.h>
-#include <pango/pangocairo.h>		/* for find_font() */
+#include <VersionHelpers.h>
 #include "../dirent/dirent-win32.h"
 #include "../../config-win32.h"
 #else
@@ -689,74 +690,88 @@ get_sys_str (int with_cpu)
 {
 	static char verbuf[64];
 	static char winver[20];
-	OSVERSIONINFOEX osvi;
 	double mhz;
 
-	osvi.dwOSVersionInfoSize = sizeof (OSVERSIONINFOEX);
-	GetVersionEx ((OSVERSIONINFO*) &osvi);
-
-	switch (osvi.dwMajorVersion)
+	if (IsWindows8Point1OrGreater ())
 	{
-		case 5:
-			switch (osvi.dwMinorVersion)
-			{
-				case 1:
-					strcpy (winver, "XP");
-					break;
-				case 2:
-					if (osvi.wProductType == VER_NT_WORKSTATION)
-					{
-						strcpy (winver, "XP x64 Edition");
-					}
-					else
-					{
-						if (GetSystemMetrics(SM_SERVERR2) == 0)
-						{
-							strcpy (winver, "Server 2003");
-						}
-						else
-						{
-							strcpy (winver, "Server 2003 R2");
-						}
-					}
-					break;
-			}
-			break;
-		case 6:
-			switch (osvi.dwMinorVersion)
-			{
-				case 0:
-					if (osvi.wProductType == VER_NT_WORKSTATION)
-					{
-						strcpy (winver, "Vista");
-					}
-					else
-					{
-						strcpy (winver, "Server 2008");
-					}
-					break;
-				case 1:
-					if (osvi.wProductType == VER_NT_WORKSTATION)
-					{
-						strcpy (winver, "7");
-					}
-					else
-					{
-						strcpy (winver, "Server 2008 R2");
-					}
-					break;
-				case 2:
-					if (osvi.wProductType == VER_NT_WORKSTATION)
-					{
-						strcpy (winver, "8");
-					}
-					else
-					{
-						strcpy (winver, "Server 2012");
-					}
-					break;
-			}
-			break;
+		if (IsWindowsServer ())
+		{
+			strcpy (winver, "Server 2012 R2");
+		}
+		else
+		{
+			strcpy (winver, "8.1");
+		}
+	}
+	else if (IsWindows8OrGreater ())
+	{
+		if (IsWindowsServer ())
+		{
+			strcpy (winver, "Server 2012");
+		}
+		else
+		{
+			strcpy (winver, "8");
+		}
+	}
+	else if (IsWindows7SP1OrGreater ())
+	{
+		if (IsWindowsServer ())
+		{
+			strcpy (winver, "Server 2008 R2 SP1");
+		}
+		else
+		{
+			strcpy (winver, "7 SP1");
+		}
+	}
+	else if (IsWindows7OrGreater ())
+	{
+		if (IsWindowsServer ())
+		{
+			strcpy (winver, "Server 2008 R2");
+		}
+		else
+		{
+			strcpy (winver, "7");
+		}
+	}
+	else if (IsWindowsVistaSP2OrGreater ())
+	{
+		if (IsWindowsServer ())
+		{
+			strcpy (winver, "Server 2008 SP2");
+		}
+		else
+		{
+			strcpy (winver, "Vista SP2");
+		}
+	}
+	else if (IsWindowsVistaSP1OrGreater ())
+	{
+		if (IsWindowsServer ())
+		{
+			strcpy (winver, "Server 2008 SP1");
+		}
+		else
+		{
+			strcpy (winver, "Vista SP1");
+		}
+	}
+	else if (IsWindowsVistaOrGreater ())
+	{
+		if (IsWindowsServer ())
+		{
+			strcpy (winver, "Server 2008");
+		}
+		else
+		{
+			strcpy (winver, "Vista");
+		}
+	}
+	else
+	{
+		strcpy (winver, "Unknown");
 	}
 
 	mhz = get_mhz ();
@@ -1253,7 +1268,7 @@ country (char *hostname)
 	domain_t *dom;
 
 	if (!hostname || !*hostname || isdigit ((unsigned char) hostname[strlen (hostname) - 1]))
-		return _("Unknown");
+		return NULL;
 	if ((p = strrchr (hostname, '.')))
 		p++;
 	else
@@ -1263,7 +1278,7 @@ country (char *hostname)
 						sizeof (domain_t), country_compare);
 
 	if (!dom)
-		return _("Unknown");
+		return NULL;
 
 	return _(dom->country);
 }
@@ -1404,7 +1419,10 @@ util_exec (const char *cmd)
 #ifndef WIN32
 	pid = fork ();
 	if (pid == -1)
+	{
+		free (argv);
 		return -1;
+	}
 	if (pid == 0)
 	{
 		/* Now close all open file descriptors except stdin, stdout and stderr */
@@ -1458,7 +1476,7 @@ make_ping_time (void)
 	GTimeVal timev;
 	g_get_current_time (&timev);
 #endif
-	return (timev.tv_sec - 50000) * 1000000 + timev.tv_usec;
+	return (timev.tv_sec - 50000) * 1000 + timev.tv_usec/1000;
 }
 
 
@@ -1881,64 +1899,6 @@ unity_mode ()
 	return 0;
 }
 
-/* Routine for listing subfolders of a given folder. ALWAYS free correctly after use, e.g.
-void display_list (GSList *list)
-{
-	GSList *iterator = NULL;
-	for (iterator = list; iterator; iterator = iterator->next)
-	{
-		printf ("%s\t", (char *) iterator->data);
-	}
-}
-
-int main (int argc, char *argv[])
-{
-	GSList *list;
-	list = get_subdirs ("foo");
-	display_list (list);
-#if GLIB_CHECK_VERSION(2,28,0)
-	g_slist_free_full (list, (GDestroyNotify) g_free);
-#else
-	g_slist_foreach (list, (GFunc) g_free, NULL);
-	g_slist_free (list);
-#endif
-	return 0;
-}
-*/
-GSList *
-get_subdirs (const char *path)
-{
-	DIR *dir;
-	struct dirent *entry;
-	GSList *dirlist = NULL;
-
-	if (!path)
-	{
-		path = ".";
-	}
-
-	dir = opendir (path);
-
-	if (!dir)
-	{
-		return NULL;
-	}
-
-	entry = readdir (dir);
-
-	while (entry != NULL)
-	{
-		if (entry->d_type == DT_DIR && strcmp (entry->d_name, ".") != 0 && strcmp (entry->d_name, "..") != 0)
-		{
-			dirlist = g_slist_append (dirlist, g_strdup (entry->d_name));
-		}
-
-		entry = readdir (dir);
-	}
-
-	return dirlist;
-}
-
 char *
 encode_sasl_pass_plain (char *user, char *pass)
 {
@@ -2179,37 +2139,6 @@ end:
 }
 #endif
 
-#ifdef WIN32
-int
-find_font (const char *fontname)
-{
-	int i;
-	int n_families;
-	const char *family_name;
-	PangoFontMap *fontmap;
-	PangoFontFamily *family;
-	PangoFontFamily **families;
-
-	fontmap = pango_cairo_font_map_get_default ();
-	pango_font_map_list_families (fontmap, &families, &n_families);
-
-	for (i = 0; i < n_families; i++)
-	{
-		family = families[i];
-		family_name = pango_font_family_get_name (family);
-
-		if (!g_ascii_strcasecmp (family_name, fontname))
-		{
-			g_free (families);
-			return 1;
-		}
-	}
-
-	g_free (families);
-	return 0;
-}
-#endif
-
 #ifdef USE_OPENSSL
 static char *
 str_sha256hash (char *string)
@@ -2287,3 +2216,96 @@ challengeauth_response (char *username, char *password, char *challenge)
 	return (char *) digest;
 }
 #endif
+
+/**
+ * \brief Wrapper around strftime for Windows
+ *
+ * Prevents crashing when using an invalid format by escaping them.
+ *
+ * Behaves the same as strftime with the addition that
+ * it returns 0 if the escaped format string is too large.
+ *
+ * Based upon work from znc-msvc project.
+ *
+ * This assumes format is a locale-encoded string. For utf-8 strings, use strftime_utf8
+ */
+size_t
+strftime_validated (char *dest, size_t destsize, const char *format, const struct tm *time)
+{
+#ifndef WIN32
+	return strftime (dest, destsize, format, time);
+#else
+	char safe_format[64];
+	const char *p = format;
+	int i = 0;
+
+	if (strlen (format) >= sizeof(safe_format))
+		return 0;
+
+	memset (safe_format, 0, sizeof(safe_format));
+
+	while (*p)
+	{
+		if (*p == '%')
+		{
+			int has_hash = (*(p + 1) == '#');
+			char c = *(p + (has_hash ? 2 : 1));
+
+			if (i >= sizeof (safe_format))
+				return 0;
+
+			switch (c)
+			{
+			case 'a': case 'A': case 'b': case 'B': case 'c': case 'd': case 'H': case 'I': case 'j': case 'm': case 'M':
+			case 'p': case 'S': case 'U': case 'w': case 'W': case 'x': case 'X': case 'y': case 'Y': case 'z': case 'Z':
+			case '%':
+				/* formatting code is fine */
+				break;
+			default:
+				/* replace bad formatting code with itself, escaped, e.g. "%V" --> "%%V" */
+				g_strlcat (safe_format, "%%", sizeof(safe_format));
+				i += 2;
+				p++;
+				break;
+			}
+
+			/* the current loop run will append % (and maybe #) and the next one will do the actual char. */
+			if (has_hash)
+			{
+				safe_format[i] = *p;
+				p++;
+				i++;
+			}
+			if (c == '%')
+			{
+				safe_format[i] = *p;
+				p++;
+				i++;
+			}
+		}
+
+		if (*p)
+		{
+			safe_format[i] = *p;
+			p++;
+			i++;
+		}
+	}
+
+	return strftime (dest, destsize, safe_format, time);
+#endif
+}
+
+/**
+ * \brief Similar to strftime except it works with utf-8 formats, since strftime treats the format as locale-encoded.
+ */
+gsize
+strftime_utf8 (char *dest, gsize destsize, const char *format, time_t time)
+{
+	gsize result;
+	GDate* date = g_date_new ();
+	g_date_set_time_t (date, time);
+	result = g_date_strftime (dest, destsize, format, date);
+	g_date_free (date);
+	return result;
+}
