@@ -27,6 +27,9 @@
 #include "fe-gtk.h"
 
 #include <gdk/gdkkeysyms.h>
+#if defined (WIN32) || defined (__APPLE__)
+#include <pango/pangocairo.h>
+#endif
 
 #include "../common/hexchat.h"
 #include "../common/fe.h"
@@ -39,10 +42,6 @@
 
 #ifdef WIN32
 #include <io.h>
-#if 0	/* native file dialogs */
-#include "../common/fe.h"
-#include "../common/thread.h"
-#endif
 #else
 #include <unistd.h>
 #endif
@@ -57,15 +56,6 @@ struct file_req
 	void *userdata;
 	filereqcallback callback;
 	int flags;		/* FRF_* flags */
-
-#if 0	/* native file dialogs */
-#ifdef WIN32
-	int multiple;
-	thread *th;
-	char *title;	/* native locale */
-	char *filter;
-#endif
-#endif
 };
 
 static void
@@ -264,7 +254,7 @@ gtkutil_esc_destroy (GtkWidget * win, GdkEventKey * key, gpointer userdata)
 	else
 		wid = win;
 
-	if (key->keyval == GDK_Escape)
+	if (key->keyval == GDK_KEY_Escape)
 		gtk_widget_destroy (wid);
 			
 	return FALSE;
@@ -325,8 +315,9 @@ fe_get_str (char *msg, char *def, void *callback, void *userdata)
 										GTK_STOCK_CANCEL, GTK_RESPONSE_REJECT,
 										GTK_STOCK_OK, GTK_RESPONSE_ACCEPT,
 										NULL);
-	gtk_box_set_homogeneous (GTK_BOX (GTK_DIALOG (dialog)->vbox), TRUE);
+
 	gtk_window_set_transient_for (GTK_WINDOW (dialog), GTK_WINDOW (parent_window));
+	gtk_box_set_homogeneous (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (dialog))), TRUE);
 
 	if (userdata == (void *)1)	/* nick box is usually on the very bottom, make it centered */
 	{
@@ -354,7 +345,7 @@ fe_get_str (char *msg, char *def, void *callback, void *userdata)
 	g_signal_connect (G_OBJECT (dialog), "response",
 						   G_CALLBACK (gtkutil_get_str_response), entry);
 
-	gtk_container_add (GTK_CONTAINER (GTK_DIALOG (dialog)->vbox), hbox);
+	gtk_container_add (GTK_CONTAINER (gtk_dialog_get_content_area (GTK_DIALOG (dialog))), hbox);
 
 	gtk_widget_show_all (dialog);
 }
@@ -383,6 +374,28 @@ gtkutil_get_number_response (GtkDialog *dialog, gint arg1, gpointer spin)
 	}
 }
 
+static void
+gtkutil_get_bool_response (GtkDialog *dialog, gint arg1, gpointer spin)
+{
+	void (*callback) (int value, void *user_data);
+	void *user_data;
+
+	callback = g_object_get_data (G_OBJECT (dialog), "cb");
+	user_data = g_object_get_data (G_OBJECT (dialog), "ud");
+
+	switch (arg1)
+	{
+	case GTK_RESPONSE_REJECT:
+		callback (0, user_data);
+		gtk_widget_destroy (GTK_WIDGET (dialog));
+		break;
+	case GTK_RESPONSE_ACCEPT:
+		callback (1, user_data);
+		gtk_widget_destroy (GTK_WIDGET (dialog));
+		break;
+	}
+}
+
 void
 fe_get_int (char *msg, int def, void *callback, void *userdata)
 {
@@ -397,7 +410,7 @@ fe_get_int (char *msg, int def, void *callback, void *userdata)
 										GTK_STOCK_CANCEL, GTK_RESPONSE_REJECT,
 										GTK_STOCK_OK, GTK_RESPONSE_ACCEPT,
 										NULL);
-	gtk_box_set_homogeneous (GTK_BOX (GTK_DIALOG (dialog)->vbox), TRUE);
+	gtk_box_set_homogeneous (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (dialog))), TRUE);
 	gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_MOUSE);
 	gtk_window_set_transient_for (GTK_WINDOW (dialog), GTK_WINDOW (parent_window));
 
@@ -408,9 +421,9 @@ fe_get_int (char *msg, int def, void *callback, void *userdata)
 
 	spin = gtk_spin_button_new (NULL, 1, 0);
 	adj = gtk_spin_button_get_adjustment ((GtkSpinButton*)spin);
-	adj->lower = 0;
-	adj->upper = 1024;
-	adj->step_increment = 1;
+	gtk_adjustment_set_lower (adj, 0);
+	gtk_adjustment_set_upper (adj, 1024);
+	gtk_adjustment_set_step_increment (adj, 1);
 	gtk_adjustment_changed (adj);
 	gtk_spin_button_set_value ((GtkSpinButton*)spin, def);
 	gtk_box_pack_end (GTK_BOX (hbox), spin, 0, 0, 0);
@@ -421,7 +434,36 @@ fe_get_int (char *msg, int def, void *callback, void *userdata)
 	g_signal_connect (G_OBJECT (dialog), "response",
 						   G_CALLBACK (gtkutil_get_number_response), spin);
 
-	gtk_container_add (GTK_CONTAINER (GTK_DIALOG (dialog)->vbox), hbox);
+	gtk_container_add (GTK_CONTAINER (gtk_dialog_get_content_area (GTK_DIALOG (dialog))), hbox);
+
+	gtk_widget_show_all (dialog);
+}
+
+void
+fe_get_bool (char *title, char *prompt, void *callback, void *userdata)
+{
+	GtkWidget *dialog;
+	GtkWidget *prompt_label;
+	extern GtkWidget *parent_window;
+
+	dialog = gtk_dialog_new_with_buttons (title, NULL, 0,
+		GTK_STOCK_NO, GTK_RESPONSE_REJECT,
+		GTK_STOCK_YES, GTK_RESPONSE_ACCEPT,
+		NULL);
+	gtk_box_set_homogeneous (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (dialog))), TRUE);
+	gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_MOUSE);
+	gtk_window_set_transient_for (GTK_WINDOW (dialog), GTK_WINDOW (parent_window));
+
+
+	g_object_set_data (G_OBJECT (dialog), "cb", callback);
+	g_object_set_data (G_OBJECT (dialog), "ud", userdata);
+
+	prompt_label = gtk_label_new (prompt);
+
+	g_signal_connect (G_OBJECT (dialog), "response",
+		G_CALLBACK (gtkutil_get_bool_response), NULL);
+
+	gtk_container_add (GTK_CONTAINER (gtk_dialog_get_content_area (GTK_DIALOG (dialog))), prompt_label);
 
 	gtk_widget_show_all (dialog);
 }
@@ -449,10 +491,6 @@ gtkutil_button (GtkWidget *box, char *stock, char *tip, void *callback,
 		gtk_widget_show (bbox);
 
 		img = gtk_image_new_from_stock (stock, GTK_ICON_SIZE_MENU);
-		if (strcmp (stock, GTK_STOCK_GOTO_LAST) == 0)
-		{
-			gtk_widget_set_usize (img, 10, 6);
-		}
 		gtk_container_add (GTK_CONTAINER (bbox), img);
 		gtk_widget_show (img);
 		gtk_box_pack_start (GTK_BOX (box), wid, 0, 0, 0);
@@ -462,7 +500,7 @@ gtkutil_button (GtkWidget *box, char *stock, char *tip, void *callback,
 							G_CALLBACK (callback), userdata);
 	gtk_widget_show (wid);
 	if (tip)
-		add_tip (wid, tip);
+		gtk_widget_set_tooltip_text (wid, tip);
 
 	return wid;
 }
@@ -479,7 +517,8 @@ GtkWidget *
 gtkutil_entry_new (int max, GtkWidget * box, void *callback,
 						 gpointer userdata)
 {
-	GtkWidget *entry = gtk_entry_new_with_max_length (max);
+	GtkWidget *entry = gtk_entry_new ();
+	gtk_entry_set_max_length (GTK_ENTRY (entry), max);
 	gtk_container_add (GTK_CONTAINER (box), entry);
 	if (callback)
 		g_signal_connect (G_OBJECT (entry), "changed",
@@ -488,89 +527,10 @@ gtkutil_entry_new (int max, GtkWidget * box, void *callback,
 	return entry;
 }
 
-GtkWidget *
-gtkutil_clist_new (int columns, char *titles[],
-						 GtkWidget * box, int policy,
-						 void *select_callback, gpointer select_userdata,
-						 void *unselect_callback,
-						 gpointer unselect_userdata, int selection_mode)
-{
-	GtkWidget *clist, *win;
-
-	win = gtk_scrolled_window_new (0, 0);
-	gtk_container_add (GTK_CONTAINER (box), win);
-	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (win),
-											  GTK_POLICY_AUTOMATIC, policy);
-	gtk_widget_show (win);
-
-	if (titles)
-		clist = gtk_clist_new_with_titles (columns, titles);
-	else
-		clist = gtk_clist_new (columns);
-
-	gtk_clist_set_selection_mode (GTK_CLIST (clist), selection_mode);
-	gtk_clist_column_titles_passive (GTK_CLIST (clist));
-	gtk_container_add (GTK_CONTAINER (win), clist);
-	if (select_callback)
-	{
-		g_signal_connect (G_OBJECT (clist), "select_row",
-								G_CALLBACK (select_callback), select_userdata);
-	}
-	if (unselect_callback)
-	{
-		g_signal_connect (G_OBJECT (clist), "unselect_row",
-								G_CALLBACK (unselect_callback), unselect_userdata);
-	}
-	gtk_widget_show (clist);
-
-	return clist;
-}
-
-int
-gtkutil_clist_selection (GtkWidget * clist)
-{
-	if (GTK_CLIST (clist)->selection)
-		return GPOINTER_TO_INT(GTK_CLIST (clist)->selection->data);
-	return -1;
-}
-
-static int
-int_compare (const int * elem1, const int * elem2)
-{
-	return (*elem1) - (*elem2);
-}
-
-int
-gtkutil_clist_multiple_selection (GtkWidget * clist, int ** rows, const int max_rows)
-{
-	int i = 0;
-	GList *tmp_clist;
-	*rows = malloc (sizeof (int) * max_rows );
-	memset( *rows, -1, max_rows * sizeof(int) );
-
-	for( tmp_clist = GTK_CLIST(clist)->selection;
-			tmp_clist && i < max_rows; tmp_clist = tmp_clist->next, i++)
-	{
-		(*rows)[i] = GPOINTER_TO_INT( tmp_clist->data );
-	}
-	qsort(*rows, i, sizeof(int), (void *)int_compare);
-	return i;
-
-}
-
-void
-add_tip (GtkWidget * wid, char *text)
-{
-	static GtkTooltips *tip = NULL;
-	if (!tip)
-		tip = gtk_tooltips_new ();
-	gtk_tooltips_set_tip (tip, wid, text, 0);
-}
-
 void
 show_and_unfocus (GtkWidget * wid)
 {
-	GTK_WIDGET_UNSET_FLAGS (wid, GTK_CAN_FOCUS);
+	gtk_widget_set_can_focus (wid, FALSE);
 	gtk_widget_show (wid);
 }
 
@@ -621,7 +581,7 @@ gtkutil_copy_to_clipboard (GtkWidget *widget, GdkAtom selection,
 	GtkClipboard *clip, *clip2;
 
 	win = gtk_widget_get_toplevel (GTK_WIDGET (widget));
-	if (GTK_WIDGET_TOPLEVEL (win))
+	if (gtk_widget_is_toplevel (win))
 	{
 		int len = strlen (str);
 
@@ -743,3 +703,33 @@ gtkutil_treeview_get_selected (GtkTreeView *view, GtkTreeIter *iter_ret, ...)
 	return has_selected;
 }
 
+#if defined (WIN32) || defined (__APPLE__)
+gboolean
+gtkutil_find_font (const char *fontname)
+{
+	int i;
+	int n_families;
+	const char *family_name;
+	PangoFontMap *fontmap;
+	PangoFontFamily *family;
+	PangoFontFamily **families;
+
+	fontmap = pango_cairo_font_map_get_default ();
+	pango_font_map_list_families (fontmap, &families, &n_families);
+
+	for (i = 0; i < n_families; i++)
+	{
+		family = families[i];
+		family_name = pango_font_family_get_name (family);
+
+		if (!g_ascii_strcasecmp (family_name, fontname))
+		{
+			g_free (families);
+			return TRUE;
+		}
+	}
+
+	g_free (families);
+	return FALSE;
+}
+#endif
